/**
 * 
 */
package sim.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

/**
 * La classe <b>SBrowser</b> reprsente un explorateur de rpertoire et de fichier.
 * 
 * @author Simon Vzina
 * @since 2018-02-26
 * @version 2018-03-14
 */
public class SBrowser {

  /**
   * La constante <b>DEFAULT_MAX_DEPTH</b> reprsente le niveau de profondeur maximal de recherche rcursive dans les sous-rpertoire.
   */
  private final int DEFAULT_MAX_DEPTH = 8;
  
  /**
   * La variable <b>file</b> reprsente l'emplacement courant de l'explorateur.
   */
  private File file;
  
  /**
   * Constructeur d'un explorateur de rpertoire.
   */
  public SBrowser()
  {
    this.file = new File(System.getProperty("user.dir"));
  }
  
  /**
   * Mthode pour obtenir le nom du rpertoire o est situ l'explorateur de rpertoire.
   * 
   * @return
   */
  public String getDirectoryName()
  {
    return this.file.getName();
  }
  
  /**
   * Mthode pour obtenir le chemin o est situ l'explorateur de rpertoire.
   * 
   * @return Le chemin de localisation de l'explorateur.
   */
  public String getPath()
  {
    return this.file.getPath();
  }
  
  /**
   * Mthode qui retourne une liste d'accs au fichier visible par la localisation courant de l'explorateur.
   * 
   * @return Un liste d'accs aux fichiers.
   */
  public List<File> getFileList()
  {
    List<File> list = new ArrayList<File>();
    
    for(File f : file.listFiles())
      if(f.isFile())
        list.add(f);
    
    return list;
  }
  
  /**
   * Mthode qui retourne une liste d'accs au rpertoire visible par la localisation courant de l'explorateur.
   * 
   * @return Un liste d'accs aux rpertoires.
   */
  public List<File> getDirectoryList()
  {
    List<File> list = new ArrayList<File>();
    
    for(File f : file.listFiles())
      if(f.isDirectory())
        list.add(f);
    
    return list;
  }
  
  /**
   * Mthode pour entrer dans un rpertoire.
   * 
   * @param directory_name Le nom du rpertoire dans lequel on veut entrer.
   * @throws FileNotFoundException Si le nom du rpertoire n'a pas t trouv.
   * @throws SManyFilesFoundException Si plusieurs rpertoires ayant le mme nom a t trouvs.
   */
  public void enter(String directory_name) throws FileNotFoundException, SManyFilesFoundException
  {
    List<File> list = getDirectoryList();
    List<File> found_list = new ArrayList<File>();
    
    for(File f : list)
      if(f.getName().equalsIgnoreCase(directory_name))
        found_list.add(f);
    
    if(found_list.isEmpty())
      throw new FileNotFoundException("Erreur SBrowser 001 : Le rpertoire '" + directory_name + "' n'a pas t trouv  partir de l'emplacement initiale '" + getPath() + "'.");
    
    if(found_list.size() > 1)
      throw new SManyFilesFoundException("Erreur SBrowser 002 : Le rpertoire '" + directory_name + "' a t trouv  plusieurs reprise  partir de l'emplacement initiale '" + getPath() + "'. Voici la liste : " + found_list);
    
    // Affectation de la nouvelle position de l'explorateur.
    this.file = found_list.get(0);
  }
  
  /**
   * Mthode pour sortir d'un rpertoire.
   */
  public void back()
  {
    this.file = new File(file.getParent());
  }
  
  /**
   * Mthode pour dplacer l'explorateur  l'emplacement d'un rpertoire  partir du rpertoire courant de l'explorateur.
   * 
   * @param directory_name Le nom du rpertoire vers lequel l'explorateur se dirigera.
   * @throws FileNotFoundException Si le nom du rpertoire n'a pas t trouv.
   * @throws SManyFilesFoundException Si plusieurs rpertoires ayant le mme nom a t trouvs.
   */
  public void goTo(String directory_name) throws FileNotFoundException, SManyFilesFoundException
  {
    goTo(directory_name, DEFAULT_MAX_DEPTH);
  }
  
  /**
   * Mthode pour dplacer l'explorateur  l'emplacement d'un rpertoire  partir du rpertoire courant de l'explorateur.
   * 
   * @param directory_name Le nom du rpertoire vers lequel l'explorateur se dirigera.
   * @throws FileNotFoundException Si le nom du rpertoire n'a pas t trouv.
   * @throws SManyFilesFoundException Si plusieurs rpertoires ayant le mme nom a t trouvs.
   */
  public void goTo(String directory_name, int max_depth) throws FileNotFoundException, SManyFilesFoundException
  {
    List<File> list = new ArrayList<File>();
    
    searchDirectories(directory_name, file, 0, max_depth, list);
    
    if(list.isEmpty())
      throw new FileNotFoundException("Erreur SBrowser 003 : Le rpertoire '" + directory_name + "' n'a pas t trouv  partir de l'emplacement initiale '" + getPath() + "'.");
    
    if(list.size() > 1)
      throw new SManyFilesFoundException("Erreur SBrowser 004 : Le rpertoire '" + directory_name + "' a t trouv  plusieurs reprise  partir de l'emplacement initiale '" + getPath() + "'. Voici la liste : " + list);
    
    // Affectation de la nouvelle position de l'explorateur.
    this.file = list.get(0);
  }
  
  /**
   * Mthode pour obtenir une liste d'accs  un fichier correspondant  un nom particulier.
   * 
   * @param file_name Le nom du fichier en recherche.
   * @return Une liste de fichier ayant le nom dsir.
   */
  public List<File> searchFiles(String file_name)
  {
    List<File> list = new ArrayList<File>();
    
    searchFiles(file_name, file, 0, DEFAULT_MAX_DEPTH, list);
    
    return list;
  }
  
  /**
   * Mthode pour obtenir un accs  un fichier correspondan  un nom particulier.
   * 
   * @param file_name Le nom du fichier en recherche.
   * @return Un accs  un fichier ayant le nom dsir.
   * @throws FileNotFoundException Si le fichier n'a pas t trouv.
   * @throws SManyFilesFoundException Si plusieurs fichiers ayant le mme nom a t trouvs.
   */
  public File findFile(String file_name) throws FileNotFoundException, SManyFilesFoundException
  {
    List<File> list = searchFiles(file_name);
    
    if(list.isEmpty())
      throw new FileNotFoundException("Erreur SBrowser 003 : Le fichier '" + file_name + "' n'a pas t trouv  partir de l'emplacement initiale '" + getPath() + "'.");
    
    if(list.size() > 1)
      throw new SManyFilesFoundException("Erreur SBrowser 004 : Le fichier '" + file_name + "' a t trouv  pluseurs reprise  partir de l'emplacement initiale '" + getPath() + "'.");
    
    return list.get(0);
  }
  
  /**
   * Mthode pour faire la recherche d'un fichier dsir de faon rcursive.  chaque fois que le fichier sera trouv,
   * l'adresse du fichier sera sauvegard dans la liste.
   * 
   * @param file_name Le nom du fichier en recherche.
   * @param current_file Le rpertoire o la recherche est rendue.
   * @param depth Le niveau de profondeur courant de la rcursivit.
   * @param max_depth Le niveau de profoncteur maximal de la rcursivit.
   * @param file_list La liste des fichies trouvs.
   */
  private void searchFiles(final String file_name, final File current_file, final int depth, final int max_depth, final List<File> file_list)
  {
    // Vrification du niveau de profondeur maximal du fichier en recherche
    if(depth <= max_depth)
    {
      // Vrification si l'analyse se fait sur un rpertoire et s'il est accessible.
      if(current_file.isDirectory() && current_file.canRead())                            
      {
        // Itrer sur l'ensemble du contenu du rpertoire
        for(File temp : current_file.listFiles())                                 
        {
          // Vrifier si le fichier itr est un rpertoire. Dans ce cas, on continue la recherche rcursivement.  
          if(temp.isDirectory())                                         
            searchFiles(file_name, temp, depth + 1, max_depth, file_list); 
          else 
            if(temp.getName().equalsIgnoreCase(file_name))                // Si le fichier est trouv ! 
                file_list.add(temp);    
        }
      } 
    }
  }
  
  /**
   * Mthode pour faire la recherche du rpertoire dsir de faon rcursive.  chaque fois que le rpertoire sera trouv,
   * l'adresse du rpertoire sera sauvegard dans la liste.
   * 
   * @param file Le rpertoire o la recherche est rendue.
   * @param depth Le niveau de profondeur de la rcursivit.
   */
  private void searchDirectories(final String directory_name, final File current_file, final int depth, final int max_depth, final List<File> directory_list)
  {
    // Vrification du niveau de profondeur maximal du fichier en recherche
    if(depth <= max_depth)
    {
      // Vrification si l'analyse se fait sur un rpertoire et l'accs au rpertoire est possible.
      if(current_file.isDirectory() && current_file.canRead())                                                  
      {                                            
        // Vrifier si le rpertoire courant porte le nom recherch.
        if(directory_name.equalsIgnoreCase(current_file.getName()))
          directory_list.add(current_file);
        
        // Itrer sur l'ensemble du contenu du rpertoire et itrer rcursivement dans le cas d'un rpertoire.
        for(File temp : current_file.listFiles())   
          if(temp.isDirectory())            
            searchDirectories(directory_name, temp, depth + 1, max_depth, directory_list);        
      }
    }
  }
  
}
