/**
 * 
 */
package sim.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import sim.exception.SConstructorException;

/**
 * La classe <b>SDirectorySearch</b> permet la recherche d'un nom de <u>rpertoire</u>  partir d'un emplacement dans un ensemble de rpertoire 
 * et de sous-rpertoire de faon rcursif.  
 * 
 * @author Simon Vzina
 * @since 2018-02-22
 * @version 2018-02-22
 */
public class SDirectorySearch {

  //-------------
  // CONSANTES //
  //-------------
  
  /**
   * La constante <b>DEFAULT_MAX_DEPTH<b> correspond au niveau de profondeur maximal de recherche par rcursivit tant gal  {@value}.
   */
  private final static int DEFAULT_MAX_DEPTH = 8;
  
//-------------
  // VARIABLES //
  //-------------
  
  /**
   * La variable <b>max_depth<b> correspond au niveau de profondeur de recherche par rcursivit.
   */
  private final int max_depth;
  
  /**
   * La variable <b>directory_name_to_search</b> correspond au rpertoire  recherche. Ce nom peut comprendre "rpertoire de localisation"/"nom du rpertoire".
   * S'il y a un nom de rpertoire d'inclu au nom du rpertoire, cette information sera ajoute  la variablle 'starting_subdirectory' lors de la recherche.
   */
  private final String directory_name_to_search;     
  
  /**
   * La variable <b>starting_subdirectory</b> correspond au nom de rpertoire  partir du quel nous allons dbuter la recherche du fichier.
   * Il est important de prciser que la recherche commencera <b>toujours</b>  partir du rpertoire <b>"user.dir"</b>.
   */
  private final String starting_subdirectory;  
  
  /**
   * La variable <b>directory_found_list</b> correspond  la liste des noms de rpertoire (rpertoire/rpertoire) o le rpertoire en recherche a t trouv.
   */
  private final List<String> directory_found_list;    
 
  /**
   * Constructeur d'un chercheur de fichier  partir du rpertoire "user.dir" o l'application est lance.
   *
   * @param directory_name Le nom du rpertoire  chercher.
   * @throws SConstructorException Si le sous-rpertoire ne permet pas d'identifier un rpertoire valide.
   * @see System.getProperty("user.dir")
   */
  public SDirectorySearch(String directory_name) throws SConstructorException
  {
    this("",directory_name);
  }
  
  /**
   * Constructeur d'un chercheur de fichier  partir du rpertoire "user.dir" o l'application est lance
   * et d'un sous-rpertoire. Le recherche par rcursivit aura une profondeur maximal par dfaut.
   * 
   * @param subdirectory Le nom du sous-rpertoire o dbutera la recherche  partir du rpertoire "user.dir". 
   * @param directory_name Le nom du rpertoire  chercher.
   * @throws SConstructorException Si le sous-rpertoire ne permet pas d'identifier un rpertoire valide.
   * @see System.getProperty("user.dir")
   */
  public SDirectorySearch(String subdirectory, String directory_name) throws SConstructorException
  {
    this(subdirectory, directory_name, DEFAULT_MAX_DEPTH);
  }
  
  /**
   * Constructeur d'un chercheur de fichier  partir du rpertoire "user.dir" o l'application est lance
   * et d'un sous-rpertoire.
   * 
   * @param subdirectory Le nom du sous-rpertoire o dbutera la recherche  partir du rpertoire "user.dir". 
   * @param directory_name Le nom du rpertoire  chercher.
   * @throws SConstructorException Si le sous-rpertoire ne permet pas d'identifier un rpertoire valide.
   * @see System.getProperty("user.dir")
   */
  public SDirectorySearch(String subdirectory, String directory_name, int depth) throws SConstructorException
  {
    // Vrification du niveau de profondeur.
    if(depth < 1)
      throw new SConstructorException("Erreur SDirectorySearch 001 : Le niveau de profondeur '" + depth + "' doit tre suprieur  0.");
    
    this.max_depth = depth;
    
    // Mettre le sous-rpertoire en minuscule
    //subdirectory = subdirectory.toLowerCase();
    
    // Obtenir le nom du fichier sans les informations de rpertoire de localisation du fichier.
    this.directory_name_to_search = SStringUtil.getFileNameWithoutDirectory(directory_name.toLowerCase());  
        
    // tablir le nom du rpertoire initiale de recherche dbutant par le rpertoire "user.dir" (l o l'application est lance)
    if(subdirectory.equals(""))
      this.starting_subdirectory = System.getProperty("user.dir");
    else
      this.starting_subdirectory = System.getProperty("user.dir") + SStringUtil.FILE_SEPARATOR_CARACTER + subdirectory;
    
    // Liste des fichiers trouvs, car il peut y avoir plusieurs fichiers avec le mme nom
    this.directory_found_list = new ArrayList<String>();        
    
    // Cration du rpertoire o dbut la recherche
    File directory = new File(starting_subdirectory); 
    
    // Vrification de la validit du nom du rpertoire
    if(directory.isDirectory())
      search(directory, 0);        //dbut de la recherche
    else
      throw new SConstructorException("Erreur SDirectorySearch 001 : Le sous-rpertoire '" + subdirectory + "' n'est pas un rpertoire dans l'adressage complet '" + starting_subdirectory + "'.");
  }
  
  /**
   * Mthode pour obtenir le nom du rpertoire  rechercher.
   * @return Le nom du rpertoire recherch.
   */
  public String getDirectoryNameToSearch()
  {
    return directory_name_to_search;
  }
 
  /**
   * Mthode pour dterminer si le rpertoire a t trouv. Cependant, il est possible que le nom du rpertoire ait t trouv plus d'une fois.
   * @return <b>true</b> si le rpertoire a t trouv et <b>false</b> sinon.
   */
  public boolean isDirectoryFound()
  {
    return !directory_found_list.isEmpty();
  }
  
  /**
   * Mthode pour dterminer si le rpertoire a t trouv <b>plus d'une fois</b>.
   * @return <b>true</b> si le rpertoire a t trouv plus d'une fois et <b>false</b> sinon. 
   */
  public boolean isManyDirectoryFound()
  {
    return directory_found_list.size() > 1;
  }
  
  /**
   * Mthode pour obtenir la liste des adresses o l'on a trouv le nom du fichier. La liste sera vide si le fichier n'a pas t trouv.
   * @return La liste des adresses o le fichier a t trouv.
   */
  public List<String> getDirectoryFoundList()
  {
    return directory_found_list;
  }
 
  
  /**
   * Mthode pour faire la recherche du rpertoire dsir de faon rcursive.  chaque fois que le rpertoire sera trouv,
   * l'adresse du rpertoire sera sauvegard dans la liste.
   * 
   * @param file Le rpertoire o la recherche est rendue.
   * @param depth Le niveau de profondeur de la rcursivit.
   */
  private void search(File file, int depth)
  {
    // Vrification du niveau de profondeur maximal du fichier en recherche
    if(depth < max_depth)
    {
      if(file.isDirectory())                                                  // Vrification si l'analyse se fait sur un rpertoire
        if(file.canRead())                                                    // Vrification si l'accs au rpertoire est possible
          if(directory_name_to_search.equals(file.getName().toLowerCase()))   // Si le rpertoire est trouv ! 
            directory_found_list.add(file.getAbsolutePath().toString());
          else
            for(File temp : file.listFiles())   // Itrer sur l'ensemble du contenu du rpertoire
              if(temp.isDirectory())            // Vrifier si le fichier itr est un rpertoire
                search(temp, depth + 1);        // Lecture rcursive du rpertoire de niveau suprieur
    }
  }
  
}// fin de la classe SDirectorySearch
